@@ -19,8 +19,8 @@ use std::fmt;
 use std::io;
 
 use hyper::Error as HttpClientError;
+use hyper::client::Response;
 use hyper::error::ParseError;
-use hyper::status::StatusCode;
 use serde_json::Error as JsonError;
 
 
@@ -34,7 +34,7 @@ pub enum ApiError {
     /// Authentication rejected (invalid credentials or token).
     Unauthorized,
     /// Generic HTTP error (not covered by EndpointNotFound and Unauthorized).
-    HttpError(StatusCode, Option<String>),
+    HttpError(Response),
     /// Protocol-level error reported by underlying HTTP library.
     ProtocolError(HttpClientError),
     /// JSON parsing failed.
@@ -50,13 +50,8 @@ impl fmt::Display for ApiError {
                 write!(f, "Requested endpoint {} was not found", endp),
             ApiError::Unauthorized =>
                 write!(f, "Authentication failed"),
-            ApiError::HttpError(status, ref maybe_msg) =>
-                match *maybe_msg {
-                    Some(ref msg) =>
-                        write!(f, "HTTP error {}: {}", status, msg),
-                    None =>
-                        write!(f, "HTTP error {}", status),
-                },
+            ApiError::HttpError(ref resp) =>
+                write!(f, "HTTP error {}", resp.status),
             ApiError::ProtocolError(ref e) => fmt::Display::fmt(e, f),
             ApiError::InvalidJson(ref e) => fmt::Display::fmt(e, f)
         }
